@@ -1,4 +1,4 @@
-# Copyright 2008 Canonical Ltd.  All rights reserved.
+# Copyright 2008-2013 Canonical Ltd.  All rights reserved.
 
 # This file is part of wadllib.
 #
@@ -732,7 +732,7 @@ class RequestDefinition(WADLBase, HasParametersMixin):
                 append = '&'
             else:
                 append = '?'
-            url += append + urlencode(validated_values)
+            url += append + urlencode(sorted(validated_values.items()))
         return url
 
 
@@ -834,7 +834,7 @@ class RepresentationDefinition(WADLResolvableDefinition, HasParametersMixin):
             params, param_values, **kw_param_values)
         media_type = self.media_type
         if media_type == 'application/x-www-form-urlencoded':
-            doc = urlencode(validated_values)
+            doc = urlencode(sorted(validated_values.items()))
         elif media_type == 'multipart/form-data':
             outer = MIMEMultipart()
             outer.set_type('multipart/form-data')
@@ -1104,13 +1104,8 @@ class Application(WADLBase):
         return ET.ElementTree(root)
 
     def _from_string(self, markup):
-        """Turns markup into a document.
-
-        Just a wrapper around ElementTree which keeps track of namespaces.
-        """
-        # We're using the ET fromstring/tostring to help make sure we maintain
-        # py2/3 compatiblity and keeping things in Unicode.
-        return self._from_stream(BytesIO(ET.tostring(ET.fromstring(markup))))
+        """Turns markup into a document."""
+        return self._from_stream(BytesIO(markup))
 
     def get_resource_type(self, resource_type_url):
         """Retrieve a resource type by the URL of its description."""
